import os
from ctypes import *
import struct
import platform
from enum import Enum, auto
from functools import wraps


def get_lib_path():
    lib_base = os.path.join(os.path.dirname(
        os.path.abspath(__file__)), '../../SDK')

    if platform.system() == 'Windows':
        if struct.calcsize("P") == 4:
            lib_name = "SECG40x86.dll"
        else:
            lib_name = "SECG40x64.dll"
    elif platform.system() == 'Linux':
        if struct.calcsize("P") == 4:
            lib_name = "libsecgx86.so"
        else:
            lib_name = "libsecgx64.so"
    elif platform.system() == 'Darwin':
        lib_name = "libsecg.a"
    return os.path.join(lib_base, lib_name)


sdk = cdll.LoadLibrary(get_lib_path())

TimeAmpCallback = CFUNCTYPE(None, c_double, c_double)


class OutputLead_E (Enum):
    Lead_RA = 0
    Lead_LA = auto()
    Lead_LL = auto()
    Lead_V1 = auto()
    Lead_V2 = auto()
    Lead_V3 = auto()
    Lead_V4 = auto()
    Lead_V5 = auto()
    Lead_V6 = auto()


class OutputFunction_E (Enum):
    Output_Off = 0
    Output_Sine = auto()
    Output_Triangle = auto()
    Output_Square = auto()
    Output_RectanglePulse = auto()
    Output_TrianglePulse = auto()
    Output_Exponential = auto()
    Output_ECG2_27 = auto()
    Output_IEC227W = auto()
    Output_IEC251W = auto()
    Output_JJG1041 = auto()
    Output_JJG1041_HR = auto()
    Output_JJG_Hysteresis = auto()
    Output_ECG_File = auto()
    Output_InputDynamicRange = auto()
    Output_ANE20000 = auto()


def sdk_fn(cfn, args_type=None, return_type=None):
    def decorator_function(func):
        cfn.argtypes = args_type
        cfn.restype = return_type

        @wraps(func)
        def wrapper(*args, **kwargs):
            return cfn(*args[1:], **kwargs)

        return wrapper

    return decorator_function


class SECG40:
    @sdk_fn(sdk.InitSECG, [], c_bool)
    def InitSECG(self) -> c_bool:
        pass

    @sdk_fn(sdk.SpecifyCOMPort, [c_int], None)
    def SpecifyCOMPort(self, port_number) -> None:
        pass

    @sdk_fn(sdk.ResetSECG, [], None)
    def ResetSECG(self) -> None:
        pass

    @sdk_fn(sdk.ResetSECG_AllLeadOff, [], None)
    def ResetSECG_AllLeadOff(self) -> None:
        pass

    def GetSerialNo(self):
        sdk.GetSerialNo.restype = c_char_p
        return sdk.GetSerialNo().decode('ascii')

    def GetHWInfo(self):
        sdk.GetHWInfo.restype = c_char_p
        return sdk.GetHWInfo().decode('ascii')

    def GetFWVersion(self):
        sdk.GetFWVersion.restype = c_char_p
        return sdk.GetFWVersion().decode('ascii')

    def GetHWVersion(self):
        sdk.GetHWVersion.restype = c_char_p
        return sdk.GetHWVersion().decode('ascii')

    def GetSDKVersion(self):
        sdk.GetSDKVersion.restype = c_char_p
        return sdk.GetSDKVersion().decode('ascii')

    @sdk_fn(sdk.RegisterTimeAmpCB, [TimeAmpCallback], None)
    def RegisterTimeAmpCB(self, cb) -> None:
        pass

    @sdk_fn(sdk.SetOutputFunc, [c_int], c_int)
    def SetOutputFunc(func) -> c_int:
        pass

    @sdk_fn(sdk.GetOutputFunc, [], c_int)
    def GetOutputFunc(self) -> c_int:
        pass

    @sdk_fn(sdk.SetAmplitude, [c_double], c_int)
    def SetAmplitude(self, amplitude) -> c_int:
        pass

    @sdk_fn(sdk.GetAmplitude, [], c_double)
    def GetAmplitude(self, amplitude) -> c_double:
        pass

    @sdk_fn(sdk.SetFrequency, [c_double], c_int)
    def SetFrequency(self, frequency) -> c_int:
        pass

    @sdk_fn(sdk.GetFrequency, [], c_double)
    def GetFrequency(self, amplitude) -> c_double:
        pass

    @sdk_fn(sdk.SetDCOffset, [c_int], c_int)
    def SetDCOffset(self, dc_offset) -> c_int:
        pass

    @sdk_fn(sdk.GetDCOffset, [], c_int)
    def GetDCOffset(self) -> c_int:
        pass

    @sdk_fn(sdk.SetDCOffsetVariableMode, [c_bool], c_int)
    def SetDCOffsetVariableMode(self, on_off) -> c_int:
        pass

    @sdk_fn(sdk.GetDCOffsetVariableMode, [], c_bool)
    def GetDCOffsetVariableMode(self) -> c_bool:
        pass

    @sdk_fn(sdk.SetDCOffsetCommonMode, [c_bool], c_int)
    def SetDCOffsetCommonMode(self, on_off) -> c_int:
        pass

    @sdk_fn(sdk.SetDCOffsetCommoneMode, [c_bool], c_int)
    def SetDCOffsetCommoneMode(self, on_off) -> c_int:
        pass

    @sdk_fn(sdk.GetDCOffsetCommonMode, [], c_bool)
    def GetDCOffsetCommonMode(self) -> c_bool:
        pass

    @sdk_fn(sdk.GetDCOffsetCommoneMode, [], c_bool)
    def GetDCOffsetCommoneMode(self) -> c_bool:
        pass

    @sdk_fn(sdk.SetOutputLead, [c_int, c_bool], c_int)
    def SetOutputLead(self, lead, on_off) -> c_int:
        pass

    @sdk_fn(sdk.GetOutputLead, [c_int], c_bool)
    def GetOutputLead(self, lead) -> c_bool:
        pass

    @sdk_fn(sdk.SetPulseWidth, [c_double], c_int)
    def SetPulseWidth(self, pulse_width) -> c_int:
        pass

    @sdk_fn(sdk.GetPulseWidth, [], c_double)
    def GetPulseWidth(self) -> c_double:
        pass

    @sdk_fn(sdk.SetInputImpedanceMode, [c_bool], c_int)
    def SetInputImpedanceMode(self, on_off) -> c_int:
        pass

    @sdk_fn(sdk.GetInputImpedanceMode, [], c_bool)
    def GetInputImpedanceMode(self) -> c_bool:
        pass

    @sdk_fn(sdk.SetPacingAmplitude, [c_double], c_int)
    def SetPacingAmplitude(self, amplitude) -> c_int:
        pass

    @sdk_fn(sdk.GetPacingAmplitude, [], c_double)
    def GetPacingAmplitude(self, amplitude) -> c_double:
        pass
